@@ -734,6 +734,59 @@ initTerminationSignals(void)
 }
 #endif /*HAVE_SIGNAL*/
 
+#ifdef O_C_STACK_GUARDED
+static void
+alt_segv_handler(int sig)
+{ GET_LD
+  (void)sig;
+
+  DEBUG(MSG_SIGNAL,
+	Sdprintf("Got C-stack overflow; critical = %d\n",
+		 LD->signal.sig_critical));
+
+  if ( LD->signal.sig_critical )
+  { longjmp(LD->signal.context, TRUE);
+    /*NORETURN*/
+  }
+
+  sigCrashHandler(sig);
+}
+#endif
+
+int
+initGuardCStack(void)
+{
+#ifdef O_C_STACK_GUARDED
+  GET_LD
+  stack_t ss = {0};
+
+  if ( (LD->signal.alt_stack = malloc(SIGSTKSZ)) )
+  { ss.ss_sp = LD->signal.alt_stack;
+    ss.ss_size = SIGSTKSZ;
+    ss.ss_flags = 0;
+
+    if ( sigaltstack(&ss, NULL) == 0)
+    { DEBUG(MSG_SIGNAL, Sdprintf("Setup altstack (%zd bytes)\n", SIGSTKSZ));
+
+      if ( LD == &PL_local_data )	/* main thread, only need this once */
+      { struct sigaction sa = {0};
+
+	sa.sa_flags = SA_ONSTACK;
+	sa.sa_handler = alt_segv_handler;
+	sigemptyset(&sa.sa_mask);
+
+	if ( sigaction(SIGSEGV, &sa, NULL) == 0 )
+	{ DEBUG(MSG_SIGNAL, Sdprintf("Setup SEGV on altstack\n"));
+	  return TRUE;
+	}
+      }
+    }
+  }
+#endif
+
+  return FALSE;
+}
+
 static void
 agc_handler(int sig)
 { GET_LD
@@ -805,6 +858,7 @@ initSignals(void)
     set_sighandler(SIGPIPE, SIG_IGN);
 #endif
     initTerminationSignals();
+    initGuardCStack();
 #endif /*HAVE_SIGNAL*/
     initBackTrace();
     for( ; sn->name; sn++)
@@ -1685,6 +1739,11 @@ freePrologLocalData(PL_local_data_t *ld)
     free_alloc_pool(ld->tabling.node_pool);
 
   clearThreadTablingData(ld);
+
+#ifdef O_C_STACK_GUARDED
+  if ( ld->signal.alt_stack )
+    free(ld->signal.alt_stack);
+#endif
 }
 
 
